import 'dart:io';

import 'container.dart';
import '../../webreflect/api.dart';

/// 每个App的实例代表一个x应用
class App {
  /// 代表连接到本应用的客户端
  List<Client> _clients = [];

  /// 应用的主页面
  Page _mainPage;

  /// 创建一个新的xApp
  App(): _mainPage = new Page();

  Page getMainPage() {
    return _mainPage;
  }

  /// 在host上启动x服务器
  start(String address, int port) async {
    var server = await HttpServer.bind(address, port);
    await for (HttpRequest req in server) {
      var socket = await WebSocketTransformer.upgrade(req);

      var client = new Client(socket);
      client.loadPage(_mainPage);
      this._clients.add(client);
    }
  }
}

class Client {
  /// 与客户端之间的wr
  WebReflect _wr;
  Page       _page;

  Client(WebSocket ws) {
    _wr = new WebReflect(ws);
    _wr.setListener(this._handler);
  }

  /// 为用户端加载一个页面
  void loadPage(Page p) {
    _page = p;
    _wr.set(WebReflect.rootId, p.generateHTML());
  }

  /// 在Client对应窗口中开一个alert
  void alert(String message) {
    _wr.executeJs('alert("$message");');
  }

  void _handler(Map<String, String> m) {
    print(m);

    var type = m['type'];
    var id = m['id'];
    var value = m['value'];
    switch (type) {
      case 'event':
        var widget = _page.getWidgetById(id);
        widget.emitEvent(value, this);
        break;
      case 'commit':
        var widget = _page.getWidgetById(id);
        widget.attr = value;
        break;
      default:
        print('未知消息类型$type'); 
    }
  }
}