import 'app.dart';

abstract class Compoent {
  /// id池
  static int _idPool = 0;

  /// 用于为组件分配id
  static int _allocId() {
    _idPool++;
    return _idPool;
  }

  Compoent():_id = _allocId();

  int _id;

  String generateHTML();
  int getId() => _id;
}

/// 窗口组件的基类
abstract class Widget extends Compoent{
  /// click事件对应名称
  static const String eClick = 'click';

  /// 组件的属性
  String attr;

  /// 组件对应html
  String html;

  /// 记录事件对应的处理器
  Map<String, Function> _eventPool = {};

  /// 用于设置id
  Widget():super();

  /// 使用event来设置事件处理函数
  /// ```
  /// button1.event('click', (var session) {
  ///   // do something..
  /// })
  /// ```
  void event(String eventName, void callback(Client)) {
    _eventPool[eventName] = callback;
  }

  void emitEvent(String eventName, Client client) {
    var callback = _eventPool[eventName];
    callback(client);
  }

  /// 生成组件对应html
  String generateHTML() {
    var html = '';

    html += '<div id="xframework-$_id">';
    html += this.html;
    html += '</div>';

    return html;
  }
}

/// 容器类型的基类
abstract class Container extends Compoent {
  List<Widget> _widgets = [];

  String _head;
  String _foot;

  /// 设置容器的head与foot以及id
  Container(this._head, this._foot): super();

  /// 添加Widegt
  void addWidgets(List<Widget> widgets) {
    _widgets.addAll(widgets);
  }

  String generateHTML() {
    var html = '';

    html += this._head;
    _widgets.forEach((Widget w) {
      html += w.generateHTML();
    });
    html += this._foot;

    return html;
  }

  Widget getWidgetById(String id) {
    for (var widget in _widgets) {
      var vid = 'xframework-' + widget.getId().toString();
      if (vid == id) {
        return widget;
      }
    }
    throw new Exception('No widget with id $id');
  }
}